unit fMain;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, XPMan, Grids, ComCtrls;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

const
  CountTrains = 116;
  CountRoadVehs = 88;
  CountShips = 11;
  CountPlanes = 41;

type
  TbtnSaveTemp = class(TForm)
    edtNfoFile: TEdit;
    lblNfoFile: TLabel;
    dlgCodecPath: TOpenDialog;
    btnNfoSave: TBitBtn;
    XPManifest1: TXPManifest;
    GroupBox1: TGroupBox;
    btnMakeGRF: TBitBtn;
    edtCodecPath: TEdit;
    btnCodecPath: TSpeedButton;
    btnNfoSelect: TBitBtn;
    PageControl1: TPageControl;
    TabSheet1: TTabSheet;
    TabSheet2: TTabSheet;
    TabSheet3: TTabSheet;
    TabSheet4: TTabSheet;
    GridTrains: TStringGrid;
    btnClear: TBitBtn;
    GridRoadVehs: TStringGrid;
    GridShips: TStringGrid;
    GridPlanes: TStringGrid;
    lblClimate: TLabel;
    cmbClimate: TComboBox;
    lblId: TLabel;
    edtID: TEdit;
    lblDescr: TLabel;
    edtDescr: TEdit;
    lblCopyright: TLabel;
    edtCopyright: TEdit;
    dlgNfoSelect: TOpenDialog;
    btnAbout: TBitBtn;
    Label1: TLabel;
    procedure btnNfoSelectClick(Sender: TObject);
    procedure btnCodecPathClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure btnNfoSaveClick(Sender: TObject);
    procedure btnMakeGRFClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure btnClearClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
  private
    FGrids: array [0..3] of TStringGrid;
  public
  end;

var
  btnSaveTemp: TbtnSaveTemp;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

implementation

uses
  ShellAPI, IniFiles,
  fAbout;

{$R *.dfm}

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function StartsStr(const AText, AStr: string): Boolean;
begin
  Result := Copy(AStr, 1, Length(AText)) = AText;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function StartsText(const AText, AStr: string): Boolean;
begin
  Result := StartsStr(UpperCase(AText), UpperCase(AStr));
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function CharReplace(const Input: string; OldChar, NewChar: Char): string;
var
  i: Integer;
begin
  Result := Input;
  for i := 1 to Length(Input) do
  begin
    if result[i] = OldChar then
      result[i] := NewChar;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function IntToHTB(const i: Integer): string;
var
  h: string;
begin
  h := IntToHex(i, 8);
  Result := Format('%s %s %s %s', [Copy(h, 7, 2), Copy(h, 5, 2), Copy(h, 3, 2), Copy(h, 1, 2)]);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function StrToHTB(const s: string): string;
var
  c: Integer;
begin
  Result := '';
  for c := 1 to Length(s) do
    Result := Result + Copy(IntToHex(Integer(Byte(s[c])), 2), 1, 2) + ' ';
  Delete(Result, Length(Result), 1);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

function HTBToStr(s: string): string;
begin
  Result := '';
  s := StringReplace(s, ' ', '', [rfReplaceAll]);
  while s <> '' do
  begin
    Result := Result + Char(StrToInt('$' + Copy(s, 1, 2)));
    Delete(s, 1, 2);
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnNfoSelectClick(Sender: TObject);
var
  lst: TStringList;
  i, p, g, n: Integer;
  FoundVersion, FoundClimate: Boolean;
  s: string;
begin
  if dlgNfoSelect.Execute then
  begin
    edtNfoFile.Text := dlgNfoSelect.FileName;
    if not FileExists(edtNfoFile.Text) then
      TFileStream.Create(edtNfoFile.Text, fmCreate).Free;
    btnClearClick(btnClear);
    try
      lst := TStringList.Create;
      try
        lst.LoadFromFile(edtNfoFile.Text);
        FoundVersion := False;
        FoundClimate := False;
        for i := 0 to lst.Count-1 do
        begin
          s := Trim(CharReplace(lst[i], #9, ' '));
          if StartsStr('1 *', s) and (Pos(' 07 83 ', s) > 5) then
          begin
            p := Pos(' 07 83 ', s) + 7;
            cmbClimate.ItemIndex := StrToInt(Copy(s, p, 2));
            FoundClimate := True;
          end
          else
          if StartsStr('2 *', s) and (Pos(' 08 02 ', s) > 5) then
          begin
            p := Pos(' 08 02 ', s) + 7;
            edtID.Text := HTBToStr(Copy(s, p, 4 * 3));
            Delete(s, 1, p + 4 * 3 - 1);
            p := Pos('00', s);
            edtDescr.Text := HTBToStr(Copy(s, 1, p - 1));
            Delete(s, 1, p + 1);
            p := Pos('00', s);
            edtCopyright.Text := HTBToStr(Copy(s, 1, p - 1));
            FoundVersion := True;
          end
          else
          if FoundClimate and FoundVersion then
          begin
            p := Pos(' 04 ', s) + 4;
            if not (p in [5..15]) then
              Continue;
            g := StrToInt('$' + Copy(s, p, 2));
            Delete(s, 1, p + 2);
            n := StrToInt('$' + Copy(s, 1, 2));
            if not (((n and 1) <> 0) or ((n and 2) <> 0)) then
              Continue;
            Delete(s, 1, 6);
            n := StrToInt('$' + Copy(s, 1, 2));
            Delete(s, 1, 3);
            p := Pos(' 00', s);
            Delete(s, p, Length(s));
            FGrids[g].Cells[1, n+1] := HTBToStr(s);
          end;
        end;
        if not FoundVersion then
          raise Exception.Create('Version/Copyright line not found')
        else
        if not FoundClimate then
          raise Exception.Create('Climate line not found')
      finally
        lst.Free;
      end;
    except
      on e: Exception do
        if not (e is EFOpenError) then
          raise;
    end;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnCodecPathClick(Sender: TObject);
begin
  if dlgCodecPath.Execute then
    edtCodecPath.Text := dlgCodecPath.FileName;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.FormCreate(Sender: TObject);
var
  i: Integer;
  g: Integer;
begin
  GridTrains.RowCount := CountTrains+1;
  GridRoadVehs.RowCount := CountRoadVehs+1;
  GridShips.RowCount := CountShips+1;
  GridPlanes.RowCount := CountPlanes+1;
  FGrids[0] := GridTrains;
  FGrids[1] := GridRoadVehs;
  FGrids[2] := GridShips;
  FGrids[3] := GridPlanes;
  for g := Low(FGrids) to High(FGrids) do
  begin
    FGrids[g].Cells[0,0] := 'ID';
    FGrids[g].Cells[1,0] := 'Vehicle Name';
    for i := 0 to FGrids[g].RowCount-2 do
    begin
      FGrids[g].Cells[0, i+1] := IntToStr(i);
    end;
  end;
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini')) do
  try
    Self.Width := ReadInteger('MainForm', 'Width', Self.Width);
    Self.Height := ReadInteger('MainForm', 'Height', Self.Height);
    Self.Left := ReadInteger('MainForm', 'Left', (Screen.Width - Self.Width) div 2);
    Self.Top := ReadInteger('MainForm', 'Top', (Screen.Height - Self.Height) div 2);
    edtCodecPath.Text := ReadString('Paths', 'CodecPath', '');
  finally
    Free;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.FormDestroy(Sender: TObject);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini')) do
  try
    WriteInteger('MainForm', 'Width', Self.Width);
    WriteInteger('MainForm', 'Height', Self.Height);
    WriteInteger('MainForm', 'Left', Self.Left);
    WriteInteger('MainForm', 'Top', Self.Top);
  finally
    Free;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  with TIniFile.Create(ChangeFileExt(Application.ExeName, '.ini')) do
    try
      WriteString('Paths', 'CodecPath', edtCodecPath.Text);
    finally
      Free;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnNfoSaveClick(Sender: TObject);
var
  lst: TStringList;
  LineOfCount: Integer;
  LinesCount: Integer;
  g, i: Integer;
begin
  lst := TStringList.Create;
  try
    lst.Add('// Automatically generated by GRFCODEC. Do not modify!');
    lst.Add('// (Info version 4)');
    lst.Add('// Format: spritenum pcxfile xpos ypos compression ysize xsize xrel yrel');
    lst.Add('');
    LineOfCount := lst.Add('');
    lst.Add(Format('1 * 6 07 83 %.2x 03 01 00', [cmbClimate.ItemIndex]));
    lst.Add(Format('2 * %d 08 02 %s %s 00 %s 00', [2 + 4 + Length(edtDescr.Text) + 1 + Length(edtCopyright.Text) + 1, StrToHTB(Copy(edtID.Text + '0000', 1, 4)), StrToHTB(edtDescr.Text), StrToHTB(edtCopyright.Text)]));
    LinesCount := 3;
    for g := Low(FGrids) to High(FGrids) do
    begin
      for i := 1 to FGrids[g].RowCount-1 do
      begin
        if FGrids[g].Cells[1, i] <> '' then
        begin
          lst.Add(Format('%d * %d 04 %.2x 1F 01 %.2x %s 00', [LinesCount, 5 + Length(FGrids[g].Cells[1, i]) + 1, g, i-1, StrToHTB(FGrids[g].Cells[1, i])])); 
          Inc(LinesCount);
        end;
      end;
    end;
    lst[LineOfCount] := '0 * 4 ' + IntToHTB(LinesCount);
    lst.SaveToFile(edtNfoFile.Text);
  finally
    lst.Free;
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnMakeGRFClick(Sender: TObject);
var
  GrfFile: string;
begin
  GrfFile := ChangeFileExt(ExtractFileName(edtNfoFile.Text), '.grf');
  if not FileExists(GrfFile) then
    TFileStream.Create(GrfFile, fmCreate).Free;
  SetCurrentDir(ExtractFilePath(edtNfoFile.Text));
  ShellExecute(0, nil, PChar('cmd.exe'), PChar(Format('/k %s -e %s .', [edtCodecPath.Text, grfFile])), PChar(ExtractFilePath(edtNfoFile.Text)), SW_SHOWNORMAL);
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnClearClick(Sender: TObject);
var
  g: Integer;
  i: Integer;
begin
  for g := Low(FGrids) to High(FGrids) do
  begin
    for i := 1 to FGrids[g].RowCount-1 do
      FGrids[g].Cells[1, i] := '';
  end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

procedure TbtnSaveTemp.btnAboutClick(Sender: TObject);
begin
  with TAboutForm.Create(Self) do
    try
      ShowModal;
    finally
      Release;
    end;
end;

{-------------------------------------------------------------------------------
-------------------------------------------------------------------------------}

end.

